#ifndef ASSERT_H
#define ASSERT_H

#define TS_BLOCK_MODE_START      0x1001
#define TS_BLOCK_MODE_FLUSH      0x1002
#define TS_BLOCK_MODE_COMMIT     0x1003
#define TS_BLOCK_MODE_COMMIT_INF 0x1004

/* assert a triple and do all the consistency checks */
int ts_assert_triple(ts_connection *c, ts_hash model, const char *subj,
                          const char *pred, const char *obj,
                          const ts_obj_type type, int dt, int lang);

/* set bulk load mode for ts_assert_triple(), pass TS_BOCK_MODE_START as action
 * before doing a lot of asserts, and TS_BOCK_MODE_COMMIT after. It's many
 * times faster */
void ts_assert_block_mode(ts_connection *db, int action);

/* make sure that a resource is in the symbols table, and return its hash */
ts_hash ts_ensure_resource(ts_connection *c, const char *uri);

/* make sure that a literal is in the symbols table, and return its hash */
ts_hash ts_ensure_literal(ts_connection *c, char *lex, int dt, int lang);

/* takes a date format string, and returns a normalised datestring in UTC if
 * possible. The timedt argument should point to a 512 char buffer that may get
 * scribbled on to store the result  */
char *ts_normalise_date(ts_connection *c, int dt, char *buffer, char *timedt);


/* Low level functions: not recommended for general use as they have no
 * consistency checks */

/* assert a raw triple, without buffering */
int ts_assert_raw(ts_connection *c, ts_hash model, ts_hash subj, ts_hash pred,
                  ts_hash obj);

/* add statements to insert val_X rows to l is not NULL, otherwise execute
 * immediatly, returns the EBV for the value */
char *ts_insert_symbol_values(ts_connection *c, GSList **l, ts_hash hash, 
			      char *lex, int dt, int lang);

/* assemble all the resource symbol inserts into a vector, dont use this
 * externally */
//ts_hash ts_block_ensure_resource(ts_connection *c, const char *uri);

#endif
