#ifndef TSP_PARSER_H
#define TSP_PARSER_H

#include <getopt.h>

#include "tstore.h"
#include "config.h"

#define TSP_OPTSTR "u:p:r:d:h"
#define TSP_LONGOPTS \
{ "username", 1, 0, 'u' }, \
{ "password", 1, 0, 'p' }, \
{ "database", 1, 0, 'd' }, \
{ "remote",   1, 0, 'r' }, \
{ "help",     0, 0, 'h' }

#define TSP_OPTIONS "[-h] [-u <user>] [-p <passwd>] [-d <db>] [-r <host>]"
#define TSP_HELP "\
built for 3store version "VERSION"\n\
   -h, --help      show help\n\
   -u, --username  set username for db connection (default '" TS_DB_USER "')\n\
   -p, --password  set password for db connection (default '" TS_DB_PASSWORD "')\n\
   -d, --database  set database for db connection (default '" TS_DB_DATABASE "')\n\
   -r, --remote    set hostname for db connection (default 'localhost')\n"

int tsp_handle_option(int opt, const char *arg);
ts_connection *tsp_connect();
ts_connection *tsp_try_connect();

void tsp_set_db(const char *db);
const char *tsp_get_user();
const char *tsp_get_pw();
const char *tsp_get_db();
const char *tsp_get_host();

#endif
