/*
 *  Copyright (C) 2005 Steve Harris
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  $Id: db-config.c,v 1.5 2005/11/16 18:46:51 theno23 Exp $
 */

#include <string.h>

#include "db-config.h"
#include "datatypes.h"
#include "util.h"
#include "config.h"

int ts_config_get_table_version(MYSQL *db)
{
    char out[TS_CF_VAL_SIZE];
    ts_connection c;

    c.db = db;
    ts_config_get_value(&c, "table_version", out);

    return atoi(out);
}

void ts_config_set_table_version(MYSQL *db, int version)
{
    char out[TS_CF_VAL_SIZE];
    ts_connection c;

    c.db = db;
    sprintf(out, "%d", version);
    ts_config_set_value(&c, "table_version", out);
}

char *ts_config_get_value(ts_connection *c, const char *key, const char *value)
{
    MYSQL_RES *res;
    MYSQL_ROW row;
    char *ret;
    char *q;

    q = g_strdup_printf("SELECT value FROM " TBLPREFIX
			"settings WHERE `key`=\"%s\"", key);
    if (mysql_query(c->db, q)) {
	free(q);
	if (mysql_errno(c->db) == 1146) {
	    return NULL;
	} else {
	    fprintf(stderr, "SQL error %d: %s\nat %s:%d\n",
                mysql_errno(c->db), mysql_error(c->db), __FILE__, __LINE__);

	    return NULL;
	}
    }
    free(q);

    res = mysql_use_result(c->db);
    row = mysql_fetch_row(res);

    if (!row) {
	return NULL;
    }

    if (value) {
	strcpy((char *)value, row[0]);
	mysql_free_result(res);

	return (char *)value;
    }

    ret = strdup(row[0]);
    mysql_free_result(res);

    return ret;
}

int ts_config_set_value(ts_connection *c, const char *key, const char *value)
{
    char *q;

    if (strlen(value) > TS_CF_VAL_LENGTH) {
	fprintf(stderr, "error: value too long in ts_config_set_value()\n");

	return 0;
    }

    q = g_strdup_printf("REPLACE INTO "TBLPREFIX"settings VALUES('%s', '%s')", key, value);
    if (mysql_query(c->db, q)) {
	fprintf(stderr, "SQL error in '%s' %d: %s\n",
		q, mysql_errno(c->db), mysql_error(c->db));

	return 0;
    }

    return 1;
}

/* vi:set ts=8 sts=4 sw=4: */
