/*
 *  Copyright (C) 2005 Steve Harris
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  $Id: hash.c,v 1.10 2005/10/03 14:15:56 theno23 Exp $
 */

#include <stdint.h>
#include <string.h>
#include <glib.h>

#include "md5.h"
#include "hash.h"

ts_hash ts_hash_uri(const char *str)
{
    uint64_t data[2];
    uint64_t top;

    if (!str) {
        return 0;
    }

    md5_buffer(str, strlen(str), data);
    top = GUINT64_FROM_BE(data[0]);

    top |= 0x8000000000000000LL;

    return top;
}

ts_hash ts_hash_literal(const char *str, uint32_t dt, uint32_t lang)
{
    uint64_t data[2];
    uint64_t top;

    if (!str) {
        return 0;
    }

    md5_buffer(str, strlen(str), data);
    top = GUINT64_FROM_BE(data[0]);

    top ^= ((ts_hash)dt) << 16;
    top ^= lang;
    top &= 0x7FFFFFFFFFFFFFFFLL;

    return top;
}

#define HASH_SIZE 4096
static ts_hash recent_h[HASH_SIZE];
static int32_t recent_i[HASH_SIZE];

void ts_symbol_cache_id(ts_hash hash, uint32_t id)
{
    recent_h[hash & (HASH_SIZE - 1)] = hash;
    recent_i[hash & (HASH_SIZE - 1)] = id;
}

ts_hash ts_symbol_id(ts_connection *c, ts_hash hash)
{
    MYSQL_RES *res;
    MYSQL_ROW row;
    char tmp[256];
    int ret;

    if (recent_h[hash & (HASH_SIZE - 1)] == hash) {
	return recent_i[hash & (HASH_SIZE - 1)];
    }

    snprintf(tmp, 256, "SELECT sid FROM symbols WHERE hash=%lld", hash);
    ts_mysql_query(c->db, tmp);
    res = mysql_store_result(c->db);
    row = mysql_fetch_row(res);
    if (!row) {
	return 0;
    }
    ret = atoi(row[0]);
    mysql_free_result(res);

    recent_h[hash & (HASH_SIZE - 1)] = hash;
    recent_i[hash & (HASH_SIZE - 1)] = ret;

    return ret;
}

int ts_ensure_datatype(ts_connection *c, const char *dt)
{
    MYSQL_RES *res;
    MYSQL_ROW row;
    char *q;

    if (!dt) {
	return 0;
    }

    q = g_strdup_printf("SELECT id FROM "T_D" WHERE uri='%s'", dt);
    mysql_query(c->db, q);
    if (mysql_errno(c->db)) {
	fprintf(stderr, "SQL error in lib3store - %s - %d\n   %s\n", q,
	    mysql_errno(c->db), mysql_error(c->db));
	return 0;
    }
    free(q);
    res = mysql_store_result(c->db);
    row = mysql_fetch_row(res);

    if (row) {
	int32_t hash = atoi(row[0]);
	mysql_free_result(res);

	return hash;
    }
    mysql_free_result(res);

    q = g_strdup_printf("INSERT INTO "T_D" VALUES(0, '%s')", dt);
    mysql_query(c->db, q);
    if (mysql_errno(c->db)) {
	fprintf(stderr, "SQL error in lib3store - %s - %d\n   %s\n", q,
	    mysql_errno(c->db), mysql_error(c->db));
    }
    free(q);

    return mysql_insert_id(c->db);
}

int ts_ensure_language(ts_connection *c, const char *lang)
{
    MYSQL_RES *res;
    MYSQL_ROW row;
    char *q;

    if (!lang) {
	return 0;
    }
    q = g_strdup_printf("SELECT id FROM "T_L" WHERE lang='%s'", lang);
    mysql_query(c->db, q);
    if (mysql_errno(c->db)) {
	fprintf(stderr, "SQL error in lib3store - %s - %d\n   %s\n", q,
	    mysql_errno(c->db), mysql_error(c->db));
	return 0;
    }
    free(q);
    res = mysql_store_result(c->db);
    row = mysql_fetch_row(res);
    mysql_free_result(res);

    if (row) {
	return atoi(row[0]);
    }

    q = g_strdup_printf("INSERT INTO "T_L" VALUES(0, '%s')", lang);
    mysql_query(c->db, q);
    if (mysql_errno(c->db)) {
	fprintf(stderr, "SQL error in lib3store - %s - %d\n   %s\n", q,
	    mysql_errno(c->db), mysql_error(c->db));
    }
    free(q);

    return mysql_insert_id(c->db);
}

char *ts_get_language(ts_connection *c, int lang_id)
{
    MYSQL_RES *res;
    MYSQL_ROW row;
    char *q;

    if (!lang_id) {
	return NULL;
    }
    q = g_strdup_printf("SELECT lang FROM "T_L" WHERE id='%d'", lang_id);
    mysql_query(c->db, q);
    if (mysql_errno(c->db)) {
	fprintf(stderr, "SQL error in lib3store - %s - %d\n   %s\n", q,
	    mysql_errno(c->db), mysql_error(c->db));
	return NULL;
    }
    free(q);
    res = mysql_store_result(c->db);
    row = mysql_fetch_row(res);
    mysql_free_result(res);

    if (row) {
	return strdup(row[0]);
    }

    return NULL;
}

char *ts_get_datatype(ts_connection *c, int id)
{
    MYSQL_RES *res;
    MYSQL_ROW row;
    char *q;

    if (!id) {
	return NULL;
    }
    q = g_strdup_printf("SELECT uri FROM "T_D" WHERE id='%d'", id);
    mysql_query(c->db, q);
    if (mysql_errno(c->db)) {
	fprintf(stderr, "SQL error in lib3store - %s - %d\n   %s\n", q,
	    mysql_errno(c->db), mysql_error(c->db));
	return NULL;
    }
    free(q);
    res = mysql_store_result(c->db);
    row = mysql_fetch_row(res);
    mysql_free_result(res);

    if (row) {
	return strdup(row[0]);
    }

    return NULL;
}

char *ts_get_resource(ts_connection *c, ts_hash hash)
{
    MYSQL_RES *res;
    MYSQL_ROW row;
    char tmp[256];

    snprintf(tmp, 255, "SELECT lexical FROM symbols WHERE hash=%lld", hash);
    ts_mysql_query(c->db, tmp);
    res = mysql_store_result(c->db);
    row = mysql_fetch_row(res);
    mysql_free_result(res);

    if (row) {
	return strdup(row[0]);
    }

    return NULL;
}

/* vi:set ts=8 sts=4 sw=4: */
