#ifndef SYMTAB_H
#define SYMTAB_H

#include <stdio.h>
#include <glib.h>

#include "tstore.h"
#include "config.h"
#include "datatypes.h"

#define TS_TYPE_TABLES 3

typedef struct ts_symtab_s {
	ts_connection *c;
	ts_query *q;
	GHashTable *hash;
	int tmpvar_cnt;
	int hash_j_cnt;
	int val_j_cnt;
	int lang_j_cnt;
	int dt_j_cnt;
	int unions;
	int union_group[TS_MAX_BLOCKS];
#ifndef ENABLE_QUADS
	int model_joins[TS_MAX_TRIPLES];
#endif /* ! ENABLE_QUADS */
} ts_symtab;

typedef struct ts_symbol_s {
	ts_symtab *st;		/* the symtab the symbol is defined in */
	char *sym;		/* the symbol name e.g. "x" */
	char *hash_col;		/* the colunm the hash value is joined on
				   e.g. triple0.subject */
	char *last_hash[TS_MAX_BLOCKS];
				/* place to store the last used hash column
				   from scoped variables */
	char *val_tbl[TS_MAX_BLOCKS];
				/* the table the symbol is joined on
				   e.g. symbol0 */
	char *tt_col;		/* the column the hash value is in in the
				   required temorary table e.g. t_r.x */
	char *lang_col;		/* the column that contains the lang tag for
				   the symbol (may be NULL) */
	char *dt_col;		/* the column that contains the dt URI for
				   the symbol (may be NULL) */
	int   block;		/* the first reqd/optional block the
				   variable appears in eg 1 */
	int   appears_in[TS_MAX_BLOCKS];
				/* true for the id of every block that
				   constains the variable */
	char *val_tbl_type[TS_TYPE_TABLES][TS_MAX_BLOCKS];
	int bound_in_msp;	/* true if this symbol is bound in the
				   subject or object slot of any triple */
} ts_symbol;

ts_symbol *ts_symbol_new(const char *sym, const char *hash_col);

void ts_symbol_free(ts_symbol *s);

const char *ts_symbol_hashcol(ts_symbol *s);

/* returns value table of symbol or NULL if not bound */
const char *ts_symbol_valtbl(ts_symbol *s, int block);

/* returns value table of symbol, will add valtbl is not allready bound */
const char *ts_symbol_valtbl_reqd(ts_symbol *s, int block);

#ifndef ENBALE_QUADS
/* returns the type table of the symbol, will join the table if required */
const char *ts_symbol_valtbl_type(ts_symbol *s, int block,
                                  rasqal_literal_type type);
#endif

int ts_symbol_appears_in(ts_symbol *s, int block);

void ts_symbol_need_lang(ts_symtab *st, ts_symbol *s);
void ts_symbol_need_dt(ts_symtab *st, ts_symbol *s);

#ifndef ENABLE_QUADS
void ts_symtab_need_model(ts_symtab *symtab, const char *tbl);
#endif /* ! ENABLE_QUADS */

#ifndef DEBUG_JOIN
#define ts_symtab_get(st, n, v, b) ts_symtab_get_intl(st, n, v, b)
#else
#define ts_symtab_get(st, n, v, b) \
ts_symtab_get_intl(st, n, v, b);\
if (v) fprintf(stderr, "val join on %s (%s) at %s:%d\n", n, ts_symtab_get_intl(st, n, v, b)->val_tbl[b], __FILE__, __LINE__);
#endif

ts_symbol *ts_symtab_get_intl(ts_symtab *symtab, const unsigned char *name, int val_needed, int block);

void ts_symtab_dump(ts_symtab *symtab, FILE *out);

ts_symtab *ts_symtab_new(ts_connection *c, ts_query *q);

void ts_symtab_free(ts_symtab *symtab);

#endif
