/*
 *  Copyright (C) 2005 Steve Harris
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  $Id: ts-hash.c,v 1.5 2005/10/11 10:07:08 theno23 Exp $
 */

#include <stdio.h>
#include <rasqal.h>
#include <string.h>
#include <libgen.h>

#include "tstore.h"
#include "cmdline-parser.h"

int main(int argc, char*argv[])
{
    ts_connection *con;
    int help = 0;
    char *optstring = TSP_OPTSTR;
    int c, opt_index = 0;
    static struct option long_options[] = {
        TSP_LONGOPTS,
	{ 0, 0, 0, 0 }
    };
    char *node;
    char *lexical, *lang, *datatype;
    ts_hash hash;

    while ((c = getopt_long (argc, argv, optstring, long_options, &opt_index)) != -1) {
        if (!tsp_handle_option(c, optarg)) {
            switch (c) {
            case 'h':
                help++;
                break;
            default:
                help++;
                break;
            }
        }
    }

    if (help || argc - optind != 1) {
        fprintf(stderr, "Usage: %s "TSP_OPTIONS" <turtle-node-string>\n" TSP_HELP, basename(argv[0]));
        return 1;
    }

    ts_init();
    con = tsp_connect(NULL);

    node = argv[optind];
    lexical = calloc(strlen(node), sizeof(char));
    lang = calloc(strlen(node), sizeof(char));
    datatype = calloc(strlen(node), sizeof(char));

    if (sscanf(node, "\"%[^\"]\"@%s", lexical, lang) == 2) {
#ifdef ENABLE_QUADS
	hash = ts_hash_literal(lexical, 0, ts_ensure_language(con,
			       (const char *)lang));
#else
	hash = ts_hash_literal(lexical, 0, ts_ensure_literal(con,
			       lang, 0, 0));
#endif
    } else if (sscanf(node, "\"%[^\"]\"^^%s", lexical, datatype) == 2) {
#ifdef ENABLE_QUADS
	hash = ts_hash_literal(lexical, ts_ensure_datatype(con,
			       (const char *)datatype), 0);
#else
	hash = ts_hash_literal(lexical, TS_ID(con, ts_ensure_resource(con,
			       (const char *)datatype)), 0);
#endif
    } else if (sscanf(node, "\"%[^\"]\"", lexical) == 1) {
	hash = ts_hash_literal(lexical, 0, 0);
    } else if (sscanf(node, "<%[^>]>", lexical)) {
	hash = ts_hash_uri((const char *)lexical);
    } else {
	fprintf(stderr, "Cannnot parse '%s' as a turtle node\n", node);

	return 1;
    }

    printf("0x%llx (%lld)\n", hash, hash);

    ts_finish();

    return 0;
}

/* vi:set ts=8 sts=4 sw=4: */
