/*
 *  Copyright (C) 2005 Steve Harris
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  $Id: ts-util.c,v 1.2 2005/09/27 16:53:30 theno23 Exp $
 */

#include <stdio.h>
#include <rasqal.h>
#include <string.h>
#include <libgen.h>

#include "tstore.h"
#include "cmdline-parser.h"
#include "config.h"

void size(ts_connection *c)
{
    int count, all, inf;
    ts_hash h, h2;
    char tmp[256];

    h = TS_ID(c, -2539357191624734300LL);
    sprintf(tmp, "SELECT COUNT(*) FROM "T_T" WHERE predicate=%lld", h);
    count = ts_sql_count(c->db, tmp);
    printf("Models                       %10d\n", count);
#ifdef ENABLE_QUADS
    all = ts_sql_count(c->db, "SELECT COUNT(*) FROM "T_T"");
#else
    all = ts_sql_count(c->db, "SELECT COUNT(*) FROM "T_M"");
#endif
    printf("All triples                  %10d\n", all);

#ifndef ENABLE_QUADS
    count = ts_sql_count(c->db, "SELECT COUNT(*) FROM "T_T);
    printf("   Distinct triples          %10d\n", count);
#endif

#ifndef ENABLE_QUADS
    snprintf(tmp, 256, "SELECT COUNT(DISTINCT mid, subject, predicate, object) FROM "T_T" t1, "T_M" t2 WHERE t2.mid=t1.subject AND t1.predicate=%lld", TS_ID(c, -9190660941158214999LL));
    inf = ts_sql_count(c->db, tmp);
#else
    inf = ts_sql_count(c->db, "SELECT COUNT(*) FROM "T_T" t1, "T_T" t2 WHERE t2.model=t1.subject AND t1.predicate=-9190660941158214999");
#endif
    printf("   Ground triples            %10d\n", all - inf);
    printf("   Inferred triples (FC)     %10d\n", inf);

    all = ts_sql_count(c->db, "SELECT COUNT(*) FROM "T_S"");
    printf("Nodes                        %10d\n", all);
    count = ts_sql_count(c->db, "SELECT COUNT(*) FROM "T_S" WHERE datatype=1");
    printf("   Resources                 %10d\n", count);
    printf("   Literals                  %10d\n", all-count);

    h = TS_ID(c, ts_hash_uri(RDF_TYPE));
    h2 = TS_ID(c, ts_hash_uri(RDFS_CLASS));
    sprintf(tmp, "SELECT COUNT(*) FROM "T_T" WHERE predicate=%lld AND object=%lld", h, h2);
    count = ts_sql_count(c->db, tmp);
    printf("Classes                      %10d\n", count);
    h2 = TS_ID(c, ts_hash_uri(RDF_PROPERTY));
    sprintf(tmp, "SELECT COUNT(*) FROM "T_T" WHERE predicate=%lld AND object=%lld", h, h2);
    count = ts_sql_count(c->db, tmp);
    printf("Properties                   %10d\n", count);
}

void deps(ts_connection *c)
{
    ts_result *res;
    ts_row *row;
    char *query = "SELECT ?x ?y "
		  "WHERE {GRAPH <"TS_SYS_MODEL"> { ?x <"INT_DEPENDS"> ?y } }";
    ts_query *tq = ts_query_prepare(c, "sparql", query, "3store:deps#");

    res = ts_query_execute(c, tq);

    while ((row = ts_result_fetch_row(res))) {
	ts_binding *x, *y;

	x = ts_row_get_binding(row, 0);
	y = ts_row_get_binding(row, 1);
	printf("%s\n   %s\n", y->lexical, x->lexical);
    }
}

void sqlconf(ts_connection *c)
{
    MYSQL_RES *res;
    MYSQL_ROW row;
    const char * const vars[] = {
	"bulk_insert_buffer_size",
	"concurrent_insert",
	"join_buffer_size",
	"key_buffer_size",
	"key_cache_block_size",
	"max_heap_table_size",
	"max_join_size",
	"myisam_data_pointer_size",
	"myisam_sort_buffer_size",
	"preload_buffer_size",
	"query_alloc_block_size",
	"query_cache_limit",
	"query_cache_min_res_unit",
	"query_cache_size",
	"query_prealloc_size",
	"read_buffer_size",
	"skip_external_locking",
	"sort_buffer_size",
	"storage_engine",
	"table_cache",
	"table_type",
	"tmp_table_size",
	"version",
	"XXXENDXXX"
    };
    int i = 0;

    ts_mysql_query(c->db, "SHOW VARIABLES");
    res = mysql_store_result(c->db);
    while ((row = mysql_fetch_row(res))) {
	if (!strcmp(row[0], vars[i])) {
	    printf("%-30s %s\n", row[0], row[1]);
	    i++;
	}
    }
    mysql_free_result(res);
}

void optimise(ts_connection *c)
{
    MYSQL_RES *res;
    MYSQL_ROW row;
    unsigned long long count;
    const char *tables[] = {
        T_T,
#ifndef ENABLE_QUADS
        T_M,
#endif
	T_S,
	T_D,
	T_L,
        NULL
    };
    unsigned int i;
    char tmp[256];

#ifndef ENABLE_QUADS
    printf("Removing stale triples... ");
    ts_orphan_cleanup(c);
    count = mysql_affected_rows(c->db);
    printf("%lld\n", count);
#endif

    printf("Removing stale symbols... ");
    fflush(stdout);
#ifdef ENABLE_QUADS
    ts_mysql_query(c->db, "DELETE FROM "T_S" WHERE NOT EXISTS (SELECT * FROM "T_T" WHERE model=hash) NOT EXISTS (SELECT * FROM "T_T" WHERE subject=hash) AND NOT EXISTS (SELECT * FROM "T_T" WHERE predicate=hash) AND NOT EXISTS (SELECT * FROM "T_T" WHERE object=hash)");
#else
    ts_mysql_query(c->db, "DELETE FROM "T_S" WHERE NOT EXISTS (SELECT * FROM "T_T" WHERE subject=sid) AND NOT EXISTS (SELECT * FROM "T_M","T_T" WHERE "T_T".tid="T_M".tid AND mid=sid) AND NOT EXISTS (SELECT * FROM "T_T" WHERE predicate=sid) AND NOT EXISTS (SELECT * FROM "T_T" WHERE object=sid)");
#endif
    count = mysql_affected_rows(c->db);
    printf("%lld\n", count);

    printf("Rebuilding indexes...\n");
    for (i = 0; tables[i]; i++) {
	snprintf(tmp, 255, "OPTIMIZE TABLE %s",tables[i]);
        if (!ts_mysql_query(c->db, tmp)) {
            res = mysql_store_result(c->db);
            while ((row = mysql_fetch_row(res))) {
		printf("%s: %s\n", row[0], row[3]);
            }
            mysql_free_result(res);
        }
    }
}

#define M_HASH_SIZE 1024

void models(ts_connection *c)
{
    ts_result *res;
    ts_row *row;
    char *query = "SELECT ?G "
		  "WHERE {GRAPH ?G { _:x _:y _:z } }";
    ts_query *tq = ts_query_prepare(c, "sparql", query, "3store:model#");
    char last[1024][512];
    int h;

    res = ts_query_execute(c, tq);

    while ((row = ts_result_fetch_row(res))) {
	ts_binding *g;

	g = ts_row_get_binding(row, 0);
	h = g_str_hash(g->lexical) & 1023;
	if (strncmp(g->lexical, last[h], 511)) {
	    snprintf(last[h], 511, "%s", g->lexical);
	    printf("%8d %s\n", h, g->lexical);
	}
    }
}

void symbols(ts_connection *c)
{
    MYSQL_RES *res;
    MYSQL_ROW row;

    ts_mysql_query(c->db, "SELECT hash, lexical FROM "T_S" LIMIT 100");
    res = mysql_use_result(c->db);
    while ((row = mysql_fetch_row(res))) {
	if (*row[0] == '-') {
	    if (*row[1] == '_') {
		printf("%s\n", row[1]);
	    } else {
		printf("<%s>\n", row[1]);
	    }
	} else {
	    printf("\"%s\"\n", row[1]);
	}
    }
    mysql_free_result(res);
}

static struct util_s {
    void (* func)(ts_connection *);
    const char * const sym;
    const char * const desc;
} util[] = {
    {deps,    "deps",    "show graph dependencies"},
    {models,  "models",  "show all model names"},
    {optimise,"optimise","compact tables for faster queries and imports"},
    {optimise,"optimize","   ditto but with US spelling..."},
    {size,    "size",    "show size of knowledge-base"},
    {sqlconf, "sqlconf", "show SQL server settings"},
    {symbols, "symbols", "show symbols (URIs and Literals)"},
    {NULL, NULL, NULL}
};

int main(int argc, char*argv[])
{
    ts_connection *con;
    int help = 0;
    char *optstring = TSP_OPTSTR;
    int c, opt_index = 0;
    static struct option long_options[] = {
        TSP_LONGOPTS,
	{ 0, 0, 0, 0 }
    };
    int i, j;
    int displayed = 0;

    while ((c = getopt_long (argc, argv, optstring, long_options, &opt_index)) != -1) {
        if (!tsp_handle_option(c, optarg)) {
            switch (c) {
            case 'h':
                help++;
                break;
            default:
                help++;
                break;
            }
        }
    }

    if (help) {
        fprintf(stderr, "Usage: %s "TSP_OPTIONS" [util-name ...]\n" TSP_HELP, basename(argv[0]));
	fprintf(stderr, "   where util-name is one of:\n");
	for (i=0; util[i].sym; i++) {
	    fprintf(stderr, "      %-8s (%s)\n", util[i].sym, util[i].desc);
	}
	fprintf(stderr, "   if none is specifed, 'size' will be used\n");
	fprintf(stderr, "See man pages for more details\n");
	    
        return 1;
    }

    ts_init();
    con = tsp_connect(NULL);

    for (i=optind; i<argc; i++) {
	int found = 0;

	for (j=0; util[j].sym; j++) {
	    if (!strcmp(argv[i], util[j].sym)) {
		if (displayed) {
		    printf("\n");
		}
		found = 1;
		displayed++;
		(util[j].func)(con);
		break;
	    }
	}
	if (!found) {
	    fprintf(stderr, "error: unknown util-name '%s'\n", argv[optind]);
	}
    }
    if (displayed == 0) {
	size(con);
    }

    ts_disconnect(con);
    ts_finish();

    return 0;
}

/* vi:set ts=8 sts=4 sw=4: */
